@@ -17,41 +17,6 @@ from solfege import mpd
 from solfege.mpd import mpdutils
 from solfege import cfg
 
-class TestRhythmStaff(unittest.TestCase):
-    def test_init(self):
-        rs = RhythmStaff()
-        rs.append(Bar(Rat(4, 4)))
-    def test_cmp(self):
-        r1 = RhythmStaff()
-        r1.append(Bar(Rat(4, 4)))
-        r2 = RhythmStaff()
-        r2.append(Bar(Rat(4, 4)))
-        self.assertEquals(r1, r2)
-        r2.append(Bar(Rat(4, 4)))
-        self.assertNotEquals(r1, r2)
-        r1.append(Bar(Rat(4, 4)))
-        self.assertEquals(r1, r2)
-        r1[-1].add_note(u"4")
-        self.assertNotEquals(r1, r2)
-        r2[-1].add_note(u"4")
-        self.assertNotEquals(r1, r2)
-
-class TestBar(unittest.TestCase):
-    def test1(self):
-        b = Bar(Rat(4, 4))
-        b.add_note(u"4")
-        b.add_note(u"4.")
-        b.add_rest(u"8")
-        b.add_note(u"4")
-        self.assertRaises(Bar.FullException, lambda: b.add_note(u"4"))
-    def test_err_1(self):
-        b = Bar(Rat(4, 4))
-        self.assertRaises(Bar.BadDuration, lambda: b.add_note("x"))
-        self.assertRaises(Bar.BadDuration, lambda: b.add_note("4,"))
-        self.assertRaises(Bar.BadDuration, lambda: b.add_note("."))
-        self.assertRaises(Bar.BadDuration, lambda: b.add_note("4.'"))
-        self.assertRaises(Bar.BadDuration, lambda: b.add_note(4))
-
 
 class TestParser(unittest.TestCase):
     def test_bad_musictype(self):
@@ -138,7 +103,7 @@ class TestChord(unittest.TestCase):
         question = self.p.m_questions[0]
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:1 v0:121 n0:48 p1:2 v1:122 n1:52 p2:3 v2:123 n2:55 d1/4 o48 o52 o55")
+            "t60/4 p0:1 p1:2 p2:3 v0:121 n0:48 v1:122 n1:52 v2:123 n2:55 d1/4 o48 o52 o55")
     def test_play_arpeggio(self):
         question = self.p.m_questions[0]
         question['music'].play_arpeggio(self.p, question)
@@ -151,7 +116,7 @@ class TestChord(unittest.TestCase):
         cfg.set_bool('config/override_default_instrument', True)
         question['music'].play_arpeggio(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(),
-            "t180/4 p0:1 v0:121 n50 d1/4 o50 p0:2 v0:122 n54 d1/4 o54 p0:3 v0:123 n57 d1/4 o57")
+            "t180/4 p0:1 p1:2 p2:3 v0:121 n50 d1/4 o50 v1:122 n54 d1/4 o54 v2:123 n57 d1/4 o57")
     def test_atonal_transpose(self):
         self.p.header.random_transpose = ('atonal', -4, 4)
         self.assertEquals(self.p.m_questions[0]['music'].get_musicdata_transposed(self.p), "d fis a")
@@ -187,7 +152,7 @@ class TestMusic(unittest.TestCase):
         question = self.p.m_questions[2]
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:37 v0:38 n0:72 p1:33 v1:34 n1:71 d1/4 o72 o71")
+            "t60/4 p0:37 p1:33 v0:38 n0:72 v1:34 n1:71 d1/4 o72 o71")
         # If the question has only one track, the last instrument,
         # representing the highest instrument, is used.
         question = self.p.m_questions[3]
@@ -211,7 +176,7 @@ class TestMusic(unittest.TestCase):
         # play with more than one instrument.:w
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:37 v0:38 n0:72 p1:35 v1:36 n1:64 p2:33 v2:34 n2:48 d1/4 o72 o64 o48")
+            "t60/4 p0:37 p1:35 p2:33 v0:38 n0:72 v1:36 n1:64 v2:34 n2:48 d1/4 o72 o64 o48")
     def test_play_slowly(self):
         question = self.p.m_questions[0]
         #test.py TestMusic viser at det ikke blir generert riktig instrument
@@ -247,12 +212,12 @@ class TestMusic3(unittest.TestCase):
         cfg.set_bool('config/override_default_instrument', True)
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:3 v0:123 n0:72 p1:2 v1:122 n1:64 p2:1 v2:121 n2:48 d1/4 o72 o64 o48")
+            "t60/4 p0:3 p1:2 p2:1 v0:123 n0:72 v1:122 n1:64 v2:121 n2:48 d1/4 o72 o64 o48")
         #
         question = self.p.m_questions[1]
         question['music'].play(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(1),
-            "t60/4 p0:37 v0:38 n0:72 p1:35 v1:36 n1:67 n1:64 p2:33 v2:34 n2:48 d1/4 o72 o67 o64 o48")
+            "t60/4 p0:37 p1:35 p2:33 v0:38 n0:72 v1:36 n1:67 n1:64 v2:34 n2:48 d1/4 o72 o67 o64 o48")
 
 class TestVoiceCommon(unittest.TestCase):
     def test_get_first_pitch(self):
@@ -270,7 +235,7 @@ class TestVoiceCommon(unittest.TestCase):
             ):
             x = Rvoice(s)
             self.assertEquals(x.get_first_pitch().get_octave_notename(), n)
-        self.assertRaises(MusicObjectException, lambda: Rvoice("\\clef bass").get_first_pitch())
+        self.assertRaises(MusicObjectException, Rvoice("\\clef bass").get_first_pitch)
 
 
 class TestVoice(LessonfileTestCase):
@@ -417,7 +382,7 @@ class TestErrorHandling(unittest.TestCase):
         cfg.set_bool('config/override_default_instrument', True)
         self._test_1()
     def _test_1(self):
-        self._dotest(("chord", "voice", "rhythm", "percussion", "rhythm"),
+        self._dotest(("chord", "voice",),
                    (("c d e ERR f",
                      "c d e ERR f\n"
                      "      ^^^"),
@@ -431,6 +396,35 @@ class TestErrorHandling(unittest.TestCase):
                      "ERR\n"
                      "^^^"),
                   ))
+    def test_barfull(self):
+        self._dotest(("music",),
+                   (("\\staff { c2. c }",
+                     "\\staff { c2. c }\n"
+                      "             ^"),
+                    ))
+        self._dotest(("voice",),
+                   (("c2. c",
+                     "c2. c\n"
+                     "    ^"),
+                    ))
+    def test_bad_clef(self):
+        self._dotest(("music",),
+                     (("\\staff { \\clef ERR",
+                      "\\staff { \\clef ERR\n"
+                        "         ^^^^^^^^^"),))
+        self._dotest(("voice", "rvoice"),
+                     (("\\clef ERR c d e",
+                       "\\clef ERR c d e\n"
+                        "^^^^^^^^^"),
+                      ("c \\clef ERR c d e",
+                       "c \\clef ERR c d e\n"
+                        "  ^^^^^^^^^"),
+                      ("\\time 4/4\n"
+                       "   c d \clef ERR3 e f\n"
+                       "  ",
+                       "   c d \clef ERR3 e f\n"
+                       "       ^^^^^^^^^^"),
+                        ))
     def test_satb(self):
         self._test_satb()
     def test_satb_3patches(self):
@@ -501,7 +495,7 @@ class TestRvoice(LessonfileTestCase):
             "\\staff\\relative c'{\nc d b c\n}")
         self.p.header.random_transpose = (True,)
         self.assertEquals(self._m().get_mpd_music_string(self.p),
-            "\\staff\\transpose d'\\relative c'{ c d b c }")
+            "\\staff\\transpose d'\\relative c'{\nc d b c\n}")
         self.p.header.random_transpose = ['atonal', -5, 6]
         self.assertEquals(self._m().get_mpd_music_string(self.p),
             "\\staff\\relative d'{\nd e cis d\n}")
@@ -544,8 +538,10 @@ class TestRvoice(LessonfileTestCase):
                 "Bad input to the music object of type rvoice made\n"
                 "the parser fail to parse the following generated\n"
                 "music code:\n"
-                "\\staff\\transpose d'\\relative e{ < e }\n"
-                "                                    ^")
+                "\\staff\\transpose d'\\relative e{\n"
+                "< e\n"
+                "}\n"
+                "^")
         else:
             self.fail("Test failed. No exception raised.")
         m = self._m(5).get_mpd_music_string(self.p)
@@ -556,8 +552,10 @@ class TestRvoice(LessonfileTestCase):
                 "Bad input to the music object of type rvoice made\n"
                 "the parser fail to parse the following generated\n"
                 "music code:\n"
-                "\\staff\\transpose d'\\relative ERR{ ERR }\n"
-                "                   ^^^^^^^^^^^^^")
+                "\\staff\\transpose d'\\relative ERR{\n"
+                "                   ^^^^^^^^^^^^^\n"
+                "ERR\n"
+                "}")
         else:
             self.fail("Test failed. No exception raised.")
         m = self._m(6).get_mpd_music_string(self.p)
@@ -568,9 +566,11 @@ class TestRvoice(LessonfileTestCase):
                 "Bad input to the music object of type rvoice made\n"
                 "the parser fail to parse the following generated\n"
                 "music code:\n"
-                "\\staff\\transpose d'\\relative c{ c d e f\n"
-                "g ERR a }\n"
-                "  ^^^")
+                "\\staff\\transpose d'\\relative c{\n"
+                "c d e f\n"
+                "g ERR a\n"
+                "  ^^^\n"
+                "}")
 
 class TestSatb(unittest.TestCase):
     def setUp(self):
@@ -622,7 +622,7 @@ class TestRhythm(unittest.TestCase):
     def setUp(self):
         cfg.set_bool('config/override_default_instrument', False)
         self.p = QuestionsLessonfile()
-        self.p.parse_string("""
+        self.p.parse_string(r"""
         question { music = rhythm("d c8") }
         question { music = rhythm("c \time 3\4 c d") }
         """)
@@ -634,7 +634,8 @@ class TestRhythm(unittest.TestCase):
             "t60/4 v9:100 P80 d1/4 o80 P37 d1/8 o37")
     def test_bad_time(self):
         question = self.p.m_questions[1]
-        question.music.get_mpd_music_string(self.p)
+        self.assertRaises(mpd.InvalidNotenameException,
+            question.music.get_mpd_music_string, self.p)
 
 class TestPercussion(unittest.TestCase):
     def setUp(self):
@@ -921,6 +922,8 @@ class TestLabelObject(unittest.TestCase):
         for s, v in (
             ("Imaj7-", [('I', 'maj', '7', '-')]),
             ("bIIIm7 -", [('bIII', 'm', '7', ' -')]),
+            (u"♯IIIm7 -", [(u'♯III', 'm', '7', ' -')]),
+            (u"♭IIIm7 -", [(u'♭III', 'm', '7', ' -')]),
             ("Imaj7-IIm7-V9-Imaj7", [('I', 'maj', '7', '-'),
                                      ('II', 'm', '7', '-'),
                                      ('V', '', '9', '-'),
@@ -952,8 +955,6 @@ class TestLabelObject(unittest.TestCase):
 
 suite = unittest.makeSuite(TestParser)
 suite.addTest(unittest.makeSuite(TestMpdTransposable))
-suite.addTest(unittest.makeSuite(TestRhythmStaff))
-suite.addTest(unittest.makeSuite(TestBar))
 suite.addTest(unittest.makeSuite(TestChord))
 suite.addTest(unittest.makeSuite(TestMusic))
 suite.addTest(unittest.makeSuite(TestMusic3))
